/*
 *  ScanGear MP for Linux
 *  Copyright CANON INC. 2007-2010
 *  All Rights Reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 *
 * NOTE:
 *  - As a special exception, this program is permissible to link with the
 *    libraries released as the binary modules.
 *  - If you write modifications of your own for these programs, it is your
 *    choice whether to permit this exception to apply to your modifications.
 *    If you do not wish that, delete this exception.
*/

/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */
#ifndef	__MAIN_C__
#define	__MAIN_C__

#define	_GLOBALS_

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"

#include "cnmstype.h"
#include "scanmain.h"

#include "w1.h"
#include "preference.h"
#include "save_dialog.h"
#include "child_dialog.h"

#include "errors.h"

#ifdef	__GIMP_PLUGIN_ENABLE__
	#include "scangimp.h"
#endif

int init(
		int		argc,
		char	*argv[] )
{
	GtkWidget	*widget = CNMSNULL;
	CNMSInt32	ldata, i, devNum;
	CNMSLPSTR	lpStr;
	int			ret = -1, status;
	CNMSByte	listname[PATH_MAX];

#ifdef ENABLE_NLS
	bindtextdomain( GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR );
	bind_textdomain_codeset( GETTEXT_PACKAGE, "UTF-8" );
	textdomain( GETTEXT_PACKAGE );
#endif

	gtk_set_locale();
	gtk_init( &argc, &argv );

	add_pixmap_directory (PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps");

#ifdef	__GIMP_PLUGIN_ENABLE__
	ScanGimpInit();
#endif

	main_window                = create_main_window ();
	select_model_dialog        = create_select_model_dialog ();
	
/* for calibration */
	calibration_dialog         = create_dialog_calibration();
	
	error_dialog               = create_error_dialog();
	about_dialog               = create_dialog_about();
	
	clear_preview_dialog       = create_dialog_clear_preview();
	
	/* set version and year */
	{
		GtkWidget		*widget;
	
		if( ( widget = lookup_widget( about_dialog, "label_about_scangear" ) ) == CNMSNULL ){
			DBGMSG( "[init]Can't get widget of label_about_scangear.\n" );
			goto	EXIT;
		}
		gtk_label_set_text( (GtkLabel *)widget, STR_ABOUT_SG STR_ABOUT_VER STR_ABOUT_LINUX);
		if( ( widget = lookup_widget( about_dialog, "label_about_copyright" ) ) == CNMSNULL ){
			DBGMSG( "[init]Can't get widget of label_about_copyright.\n" );
			goto	EXIT;
		}
		gtk_label_set_text( (GtkLabel *)widget, STR_ABOUT_CR STR_ABOUT_AUTHOR STR_ABOUT_YEAR );
	}

	gtk_window_set_default_size( (GtkWindow *)main_window, gdk_screen_width() * 0.8, gdk_screen_height() * 0.8 );

	if( ( devNum = CnmsScanInit( CNMS_TRUE ) ) < 0 ){
		goto	EXIT_ERR;
	}

	if( ( ldata = ChildDialogOpen() ) == CNMS_ERR ){
		DBGMSG( "[init]Error is occured in ChildDialogOpen.\n" );
		goto	EXIT;
	}
	else if( ( ldata = ProgressBarOpen() ) != CNMS_NO_ERR ){
		DBGMSG( "[init]Error is occured in ProgressBarOpen.\n" );
		ChildDialogClose();
		goto	EXIT;
	}

	if( ( status = W1_ShowSelectModelForNetwork( devNum ) ) == CNMS_ERR ) {
		goto EXIT_ERR;
	}

	gtk_main ();
	ret = CNMS_NO_ERR;
EXIT:
	return	ret;

EXIT_ERR:
	goto	EXIT;
}

int main(
		int		argc,
		char	*argv[] )
{
	int			ret = CNMS_ERR;
	char		*str = NULL;
	CNMSInt32	ldata;

	str = gtk_set_locale();

#ifdef	__GIMP_PLUGIN_ENABLE__
	if ( argc > 1 ) {
		ScanGimpOpen();
		if( ( ldata = ScanGimpMain( argc, argv ) ) != CNMS_NO_ERR ){
			goto	EXIT;
		}
	}
#endif
	ret = init( argc, argv );
EXIT:
#ifdef	__GIMP_PLUGIN_ENABLE__
	if ( argc > 1 ) {
		ScanGimpClose();
	}
#endif
	return	ret;
}

#endif	/* End of __MAIN_C__ */
